﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using WizardForms.Masters;
using Business;

namespace WizardForms
{
    #region Enum of wizard status
    public enum Status
    {
        Started,
        Running,
        Aborted,
        Finished
    }
    #endregion

    #region WizardStatusArgs class for event management
    public class WizardStatusArgs : EventArgs
    {
        int _currentStep;
        Status _wizardStatus;
        public Status WizardStatus
        {
            get
            {
                return _wizardStatus;
            }
            set
            {
                _wizardStatus = value;
            }
        }
        public int CurrentStep
        {
            get
            {
                return _currentStep;
            }
            set
            {
                _currentStep = value;
            }
        }
        public WizardStatusArgs(Status _status, int _step)
        {
            _currentStep = _step;
            _wizardStatus = _status;
        }
    }
    #endregion

    public delegate void WizardDelegate(object sender, WizardStatusArgs wsa);

    public class WizardController
    {
        public event WizardDelegate WizardEvent;
        
        #region variables
        private int _stepIndex = -1;
        private List<Form> _stepForms = new List<Form>();
        private Status _status;
        private Something _sumthin; //instance of the business object to track changes through the wizard
        #endregion

        #region public properties
        #endregion

        #region public methods
        #region constructor
        public WizardController()
        {
            //create the business object
            _sumthin = new Something();
        }
        #endregion
        public void AddStep(WizardMaster stepForm)
        {
            WizardMaster currentStep = stepForm;
            if (currentStep != null)
            {
                _stepForms.Add(currentStep);
                currentStep.VisibleChanged += new EventHandler(currentStep_VisibleChanged);
                if (currentStep.PreviousButton != null)
                    currentStep.PreviousButton.Click += new EventHandler(currentStep_NavigatePrevious);
                if (currentStep.NextButton != null)
                    currentStep.NextButton.Click += new EventHandler(currentStep_NavigateNext);
                if (currentStep.FinishButton != null)
                    currentStep.FinishButton.Click += new EventHandler(currentStep_NavigateFinish);
                currentStep.FormClosed += new FormClosedEventHandler(currentStep_FormClosed);
            }
            else
            {
                throw new Exception("Wizard could not add step");
            }
        }

        public void StartWizard()
        {
            if (_stepForms.Count == 0)
                throw new Exception("No steps in the wizard.");

            if (_stepIndex != -1 && _status==Status.Running)
                throw new Exception("The WizardController has already been started");

            _stepIndex = 0;
            _status = Status.Running;
            Form startForm = _stepForms[_stepIndex] as Form;
            if (startForm != null)
            {
                startForm.Show();
                WizardEvent(this, new WizardStatusArgs(_status, _stepIndex));
            }
        }
        #endregion

        #region Event Handlers
        protected void currentStep_VisibleChanged(object sender, EventArgs e)
        {
            //event fires after the visibility has changed
            //when something becomes visible we need to give the controls values
            if (((Form)sender).Visible)
                UpdateFormControls((WizardMaster)sender);
            else //when it becomes hidden, we need to get the values from the controls
                UpdateBusinessClass((WizardMaster)sender);
        }
        protected void currentStep_NavigatePrevious(object sender, EventArgs e)
        {
            if (_stepIndex > 0)
            {
                Form currentForm = _stepForms[_stepIndex] as Form;
                Form prevForm = _stepForms[_stepIndex - 1] as Form;
                if (currentForm != null && prevForm != null)
                {
                    prevForm.Location = currentForm.Location;
                    currentForm.Hide();
                    prevForm.Show();
                    _stepIndex--;
                }
            }
        }
        protected void currentStep_NavigateNext(object sender, EventArgs e)
        {
            if (_stepIndex < _stepForms.Count-1)
            {
                Form currentForm = _stepForms[_stepIndex] as Form;
                Form nextForm = _stepForms[_stepIndex + 1] as Form;
                if (currentForm != null && nextForm != null)
                {
                    currentForm.Hide();
                    nextForm.Show();
                    nextForm.Location = currentForm.Location;
                    _stepIndex++;
                }
            }
        }
        protected void currentStep_NavigateFinish(object sender, EventArgs e)
        {           
            UpdateBusinessClass((WizardMaster)((Button)sender).FindForm());
            if (!String.IsNullOrEmpty(_sumthin.Filename) && !String.IsNullOrEmpty(_sumthin.Sentence))
            {
                //do the activity
                _sumthin.DoSomething();
                _status = Status.Finished;
                WizardEvent(this, new WizardStatusArgs(_status, _stepIndex));
            }


        }
        protected void currentStep_FormClosed(object sender, EventArgs e)
        {
            _status = Status.Aborted;
            WizardEvent(this, new WizardStatusArgs(_status, _stepIndex));
        }

        #endregion

        #region private methods
        private void UpdateFormControls(WizardMaster form)
        {
            //form is being made visible, so copy values back to form
            //this is not really necessary for this example, 
            //but in theory you could have situations where values change in other parts of the 
            //wizard (or business object and need to be reflected back (or forward).
            switch (_stepIndex)
            {
                case 0:
                    StepOne step1 = form as StepOne;
                    if (step1 != null)
                        step1.FilenameText = _sumthin.Filename;
                    break;
                case 1:
                    StepTwo step2 = form as StepTwo;
                    if (step2 != null)
                        step2.SentenceText = _sumthin.Sentence;
                    break;
                case 2:
                    StepThree step3 = form as StepThree;
                    if (step3 != null)
                        step3.IterationsChosen = _sumthin.Iterations;
                    break;
            }
        }
        private void UpdateBusinessClass(WizardMaster form)
        {
            //form is being hidden, so copy values to the business object
            switch (_stepIndex)
            {
                case 0:
                    StepOne step1 = form as StepOne;
                    if (step1 != null)
                        _sumthin.Filename = step1.FilenameText;
                    break;
                case 1:
                    StepTwo step2 = form as StepTwo;
                    if (step2 != null)
                        _sumthin.Sentence = step2.SentenceText;
                    break;
                case 2:
                    StepThree step3 = form as StepThree;
                    if (step3 != null)
                        _sumthin.Iterations = step3.IterationsChosen;
                    break;
            }
        }

        #endregion
    }
}
